#
# Abbaye des Morts for Parallax Propeller
# High speed Prop-to-Prop serial communication
#
# Author: Beau Schwabe
# MIT Licensed
#

                    .pasm
                    .compress off

                    .section .cog_hsp2ptx_driver, "ax"

                    .org     0

# Start TX proceedure ≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
TX_Propeller
                    mov     DataIndex, par          ' Read PacketHeader Address
                    rdlong  Buff, DataIndex         ' Read PacketHeader contents into Buff
                    mov     DataIndex, Buff
# Parse Destination Offset and replace previous outgoing data address with new offset value
                    mov     DestOffset, par
                    add     DestOffset, #4
                    rdlong  DestOffset, DestOffset
                    shl     DestOffset, #4
                    andn    Buff, AddressMask
                    or      Buff, DestOffset
# Parse Packet Header so we know where to send the data from
                    and     DataIndex, AddressMask  ' DataIndex points to data buffer
                    shr     DataIndex, #4
# Parse Packet Header to get number of packets to send
                    mov     DataSamples, Buff
                    shr     DataSamples, #18
                    add     DataSamples, #1
# Create Pin Mask for data transmission line
                    mov     PinMask, #1             ' Create Pin Mask for Input/Output pin
                    shl     PinMask, Pin
                    or      outa, PinMask           ' Preset Pin HIGH
                    andn    dira, PinMask           ' Make Pin an INPUT Hi-Z LOW
# Setup Counter to count during data-pin HIGH times
                    movi    ctra, #%0_11010_000     ' LOGIC A
                    movs    ctra, Pin
                    mov     frqa, #1
# Send 3.3us Packet SYNC
TX_Packet_Sync
                    or      dira, PinMask           ' Make pin HIGH
                    mov     delay, #63              ' 3.3us Packet Sync
                    djnz    delay, #$
                    andn    dira, PinMask           ' Make Pin an INPUT Hi-Z LOW
                    mov     phsa, #0

                    long    0,0,0,0,0,0             ' <-- Short hand way to write 6 NOP instructions on a single line
                                                    '     NOPs provide window for RX unit to respond with a Sync pulse

                    mov     temp, phsa
                    cmp     temp, #3  wc            ' Detect if Packet Sync signal received
        if_c        jmp     #TX_Packet_Sync         ' ... temp value should equal 4, indicating
                    nop                             ' 4 clocks, but could fluxtuate depending on
                    nop                             ' slight phase differnces... we'll compare it
                    nop                             ' to a 3
                    nop
# Send SYNC ; and get first bit to send≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
_TX                 or      dira, PinMask           ' Make pin HIGH
                    rol     Buff, #1 wc           ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    andn    dira, PinMask           ' Make Pin an INPUT Hi-Z LOW
# Send 1 LONG  ≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    rol     Buff, #1 wc             ' Read Buff Bit31 into "C" ; Rotate Buff left by 1
                    muxc    dira, PinMask           ' Make Pin HIGH or LOW (100nS)  DATA
                    nop
# Null Bit ≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
                    andn    dira, PinMask           ' Make Pin an INPUT Hi-Z LOW
                    nop
                    nop
# Read next Data long ≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
                    rdlong  Buff, DataIndex         ' Read Data Value from Indexed location
                    add     DataIndex, #4           ' Increment Index value

                    long    0,0,0,0,0,0,0           ' <-- Short hand way to write 7 NOP instructions on a single line
                                                    ' <-- Optimization on the receiving end will clean up some of the NOP's

                    djnz    DataSamples, #_TX       ' Check to see if there are more data samples
# Set Status Flag ≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
                    rdlong  DataSamples, par        ' read data samples
                    or      DataSamples, #$F        ' set flag; data packet done
                    wrlong  DataSamples, par        ' update flag status
# ≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈≈
terminate           cogid   temp                    ' Get this COG ID
                    cogstop temp                    ' STOP this COG


Pin                 long    11
PinMask             long    0
Buff                long    0
DataSamples         long    0
DataIndex           long    0
temp                long    0

AddressMask         long    $7FFF0
_PacketHeaderIn     long    0
delay               long    0
DestOffset          long    0
