#
# Abbaye des Morts for Parallax Propeller
# Tile and Sprite LCD Scanline Renderer
#
# Copyright (c) 2015 Marco Maccaferri
# MIT Licensed.
#

                    .pasm
                    .compress off

                    .section .cogvideo_renderer, "ax"

                    .equ     hres, 256
                    .equ     vres, 192
                    .equ     cogs, 4

                    .org     0

driver              jmp     #begin

# driver parameters

offset              long    0                       ' 0x0001
sbuf                long    0                       ' 0x0002

begin               mov     tile_palette_ptr, #tile_palette
                    shl     tile_palette_ptr, #16
                    or      tile_palette_ptr, #tile_palette

                    mov     scnt, #vres-cogs
                    add     scnt, offset

loop
                    rdlong  a, blnk
                    cmp     a, scnt wz
            if_nz   jmp     #$-2                    ' wait for line fetch start

                    add     scnt, #cogs             ' next line to render
                    cmpsub  scnt, #vres wz,wc
            if_nc   jmp     #render

vsync
                    ' read sprite status

                    mov     sbuf_ptr, sprite_ram
                    movd    _rd0, #sprite11 +1
                    movd    _rd1, #sprite11
                    add     sbuf_ptr, #24 * 4 -1
                    movi    sbuf_ptr, #24 - 2
_rd0                rdlong  0-0, sbuf_ptr
                    sub     _rd0, inc_dest_2
                    sub     sbuf_ptr, i2s7 wc
_rd1                rdlong  0-0, sbuf_ptr
                    sub     _rd1, inc_dest_2
            if_nc   djnz    sbuf_ptr, #_rd0

render              mov     video_ptr, scnt         ' render line
                    shr     video_ptr, #3
                    shl     video_ptr, #6
                    add     video_ptr, video_ram
                    
                    mov     loffs, scnt
                    and     loffs, #7
                    shl     loffs, #2
                    add     loffs, tiles_rom

                    mov     ccnt, #0
                    mov     ecnt, #hres/8
                    mov     sbuf_ptr, sbuf

rloop               rdword  tile_ptr, video_ptr     ' read tile number to display

                    mov     palette_ptr, tile_ptr
                    and     palette_ptr, palettemask
                    shr     palette_ptr, #6

                    and     tile_ptr, tilemask
                    shl     tile_ptr, #5            ' 32 bytes per tile
                    add     tile_ptr, loffs
                    rdlong  tile, tile_ptr          ' pixels, 4 bit per pixel, from msb

                    mov     a, palette_ptr
                    shl     a, #16
                    or      palette_ptr, a
                    add     palette_ptr, tile_palette_ptr

                    mov     a, tile
                    and     a, mask_04
                    shr     a, #12
                    add     a, palette_ptr
                    movs    mv4, a
                    shr     a, #16
                    movs    mv0, a

                    mov     a, tile
                    and     a, mask_15
                    shr     a, #8
                    add     a, palette_ptr
                    movs    mv5, a
                    shr     a, #16
                    movs    mv1, a

                    mov     a, tile
                    and     a, mask_26
                    shr     a, #4
                    add     a, palette_ptr
                    movs    mv6, a
                    shr     a, #16
                    movs    mv2, a

                    and     tile, mask_37           ' last pixel pair, tile not needed anymore
                    add     tile, palette_ptr
                    movs    mv7, tile
                    shr     tile, #16
                    movs    mv3, tile

mv1                 mov     pixels0, 0-0
                    shl     pixels0, #16
mv0                 or      pixels0, 0-0
mv3                 mov     pixels1, 0-0
                    shl     pixels1, #16
mv2                 or      pixels1, 0-0
mv5                 mov     pixels2, 0-0
                    shl     pixels2, #16
mv4                 or      pixels2, 0-0

                    wrlong  pixels0, sbuf_ptr
                    add     sbuf_ptr, #4
mv7                 mov     pixels3, 0-0
                    wrlong  pixels1, sbuf_ptr
                    add     sbuf_ptr, #4
                    shl     pixels3, #16
                    wrlong  pixels2, sbuf_ptr
                    add     sbuf_ptr, #4
mv6                 or      pixels3, 0-0
                    wrlong  pixels3, sbuf_ptr
                    add     sbuf_ptr, #4

                    add     ccnt, #8
                    add     video_ptr, #2
                    djnz    ecnt, #rloop

                    cmp     scnt, #vres-16 wz,wc    ' no sprites below vres-16 (status bar)
            if_nc   jmp     #loop

                    mov     tile, sprite0
                    mov     tile_ptr, sprite0 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite1
                    mov     tile_ptr, sprite1 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite2
                    mov     tile_ptr, sprite2 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite3
                    mov     tile_ptr, sprite3 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite4
                    mov     tile_ptr, sprite4 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite5
                    mov     tile_ptr, sprite5 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite6
                    mov     tile_ptr, sprite6 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite7
                    mov     tile_ptr, sprite7 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite8
                    mov     tile_ptr, sprite8 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite9
                    mov     tile_ptr, sprite9 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite10
                    mov     tile_ptr, sprite10 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite11
                    mov     tile_ptr, sprite11 + 1 wz
            if_nz   call    #draw_sprite

                    jmp     #loop

draw_sprite         mov     y, tile                 ' y-position mask
                    shl     y, #14
                    sar     y, #23                  ' sign-extend y

                    mov     b, tile_ptr             ' calculate height
                    shr     b, #19
                    and     b, #$38

                    mov     a, scnt                 ' check sprite scanline visibility
                    subs    a, y  wc,wz
            if_c    jmp     #draw_sprite_ret
                    cmp     a, b wc,wz
            if_nc   jmp     #draw_sprite_ret

                    mov     b, tile_ptr             ' set palette pointer
                    shr     b, #25
                    mov     palette_ptr, b
                    shl     palette_ptr, #16
                    or      palette_ptr, b
                    add     palette_ptr, tile_palette_ptr

                    mov     b, tile_ptr             ' calculate width
                    shr     b, #16
                    and     b, #$38

                    shl     a, #2
                    add     tile_ptr, a
                    cmp     b, #16 wc
            if_nc   add     tile_ptr, a
            if_nc   cmp     b, #24 wc
            if_nc   add     tile_ptr, a
            if_nc   cmp     b, #32 wc
            if_nc   add     tile_ptr, a

                    mov     ecnt, b                 ' width counter
                    
                    mov     x, tile                 ' x-position mask
                    shl     x, #23
                    sar     x, #22                  ' sign-extend x * 2
                    mov     sbuf_ptr, sbuf
                    add     sbuf_ptr, x             ' adjust scanline buffer pointer to x location
                    sar     x, #1

                    test    tile, mirror_mask wz
            if_nz   jmp     #_l4m

_l2                 rdlong  tile, tile_ptr
                    add     tile_ptr, #4
                    rol     tile, #4

                    mov     ccnt, #8
_l1                 mov     a, tile
                    and     a, #$0F wz
                    add     a, palette_ptr
                    movd    _l3, a
                    cmps    x, #0 wc
_l3  if_nz_and_nc   wrword  0-0, sbuf_ptr
                    add     sbuf_ptr, #2
                    adds    x, #1
                    cmps    x, #hres wz,wc
            if_nc   jmp     #draw_sprite_ret
                    rol     tile, #4
                    sub     ecnt, #1 wz
            if_z    jmp     #draw_sprite_ret
                    djnz    ccnt, #_l1
                    jmp     #_l2

_l4m                sub     b, #8
                    shr     b, #1
                    add     tile_ptr, b

_l2m                rdlong  tile, tile_ptr
                    sub     tile_ptr, #4

                    mov     ccnt, #8
_l1m                mov     a, tile
                    and     a, #$0F wz
                    add     a, palette_ptr
                    movd    _l3m, a
                    cmps    x, #0 wc
_l3m if_nz_and_nc   wrword  0-0, sbuf_ptr
                    add     sbuf_ptr, #2
                    adds    x, #1
                    cmps    x, #hres wz,wc
            if_nc   jmp     #draw_sprite_ret
                    ror     tile, #4
                    sub     ecnt, #1 wz
            if_z    jmp     #draw_sprite_ret
                    djnz    ccnt, #_l1m
                    jmp     #_l2m

draw_sprite_ret     ret


' driver parameters

blnk                long    @_vsync
video_ram           long    @_video_ram
tiles_rom           long    @_tiles_ram
sprite_ram          long    @_sprite_ram

' initialised data and/or presets

inc_dest            long    1 << 9
inc_dest_2          long    2 << 9
i2s7                long    2 << 23 | 7
mask_04             long    $F_0_0_0_F_0_0_0
mask_15             long    $0_F_0_0_0_F_0_0
mask_26             long    $0_0_F_0_0_0_F_0
mask_37             long    $0_0_0_F_0_0_0_F
datamask            long    $0000_FFFF

tilemask            long    %0_00000_1111111111
palettemask         long    %0_11111_0000000000

mirror_mask         long    %0000001_0000000_000000000_000000000

                    long    0                       ' \
                    long    0                       '  |
pixels0             long    0                       '  | sprite pixels, space for shifting
pixels1             long    0                       '  |
pixels2             long    0                       ' /
pixels3             long    0

tile_palette        .incbin "lcd_palette.dat"

tile_palette_ptr    long    0 ' (tile_palette << 16) | tile_palette
palette_size        long    (16 << 16) | 16

' uninitialised data and/or temporaries

a                   res     1
b                   res     1
x                   res     1
y                   res     1
cmd                 res     1                       ' command
cmd_out             res     1
data                res     1                       ' data
data_out            res     1
tile                res     1
ptr                 res     1

acnt                res     1                       ' attribute count
ecnt                res     1                       ' element count
lcnt                res     1                       ' line counter
scnt                res     1                       ' scanlines
ccnt                res     1                       ' column counter
loffs               res     1                       ' line offset

sbuf_ptr            res     1
tile_ptr            res     1
sprite_ptr          res     1
video_ptr           res     1
palette_ptr         res     1

sprite0             res     2
sprite1             res     2
sprite2             res     2
sprite3             res     2
sprite4             res     2
sprite5             res     2
sprite6             res     2
sprite7             res     2
sprite8             res     2
sprite9             res     2
sprite10            res     2
sprite11            res     2

                    fit     $1F0


                    .data

                    .global _sprite_ram
                    .global _video_ram

_sprite_ram         res     12 * 2
_video_ram          res     (((hres / 8) * (vres / 8)) * 2) / 4
