/**
 * Abbaye des Morts for Parallax Propeller
 * Copyright (c) 2015 Marco Maccaferri
 *
 * TERMS OF USE: MIT License
 */

#ifndef OVERLAY_H_
#define OVERLAY_H_

struct room_data {
    uint32_t enemies_offs;
    uint32_t tiles_ram;
    uint32_t tileset_size;
    void (*drawenemies)();
    void (*touchobj)();
    void (*events)();
};

void statusbar();
void common_drawenemies();

void fireball();
void plants();
void dragon();
void death();

void redparchment();
void blueparchment();

#define TOUCH_CROSSES \
    /* Touch crosses */ \
    if (((video_ram[y+1][x] >= TILE409_ID) &&  (video_ram[y+1][x] <= TILE412_ID)) || ((video_ram[y+1][x+1] >= TILE409_ID) &&  (video_ram[y+1][x+1] <= TILE412_ID)) || ((video_ram[y+2][x] >= TILE409_ID) &&  (video_ram[y+2][x] <= TILE412_ID))) { \
        for (v=0;v<22;v++) { \
            for (h=0;h<32;h++) { \
                if ((video_ram[v][h] >= TILE409_ID) && (video_ram[v][h] <= TILE412_ID)) \
                    video_ram[v][h] = 0; \
            } \
        } \
        jean.state[1] += 1; \
        statusbar(); \
        hs_tx(sizeof(video_ram) / 4, (int32_t)video_ram, 0, GPU_VIDEO_RAM); \
        spi_write((room[0] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
    }

#define TOUCH_HEARTS \
    /* Touch hearts */ \
    if (((video_ram[y+1][x] >= TILE401_ID) &&  (video_ram[y+1][x] < TILE405_ID)) || ((video_ram[y+1][x+1] >= TILE401_ID) &&  (video_ram[y+1][x+1] < TILE405_ID))) { \
        for (v=0;v<22;v++) { \
            for (h=0;h<32;h++) { \
                if ((video_ram[v][h] >= TILE401_ID) && (video_ram[v][h] < TILE405_ID)) \
                    video_ram[v][h] = 0; \
            } \
        } \
        if (jean.state[0] < 9) { \
            jean.state[0] += 1; \
            statusbar(); \
        } \
        hs_tx(sizeof(video_ram) / 4, (int32_t)video_ram, 0, GPU_VIDEO_RAM); \
        spi_write((room[0] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
    }

#define TOUCH_CHECKPOINT \
    if (((video_ram[y][x] >= TILE321_ID) && (video_ram[y][x] <= TILE324_ID)) || ((video_ram[y][x+1] >= TILE321_ID) && (video_ram[y][x+1] <= TILE324_ID))) { \
        jean.checkpoint[3] = jean.checkpoint[0]; \
        jean.checkpoint[0] = room[0]; \
        jean.checkpoint[1] = jean.x; \
        jean.checkpoint[2] = jean.y; \
\
        /* Old checkpoint returns to original state */ \
        spi_read((jean.checkpoint[3] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
        for (v=0;v<22;v++) { \
            for (h=0;h<32;h++) { \
                if ((video_ram[v][h] >= TILE327_ID) && (video_ram[v][h] <= TILE330_ID)) \
                    video_ram[v][h] -= 6; \
            } \
        } \
        spi_write((jean.checkpoint[3] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
\
        spi_read((room[0] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
        for (v=0;v<22;v++) { \
            for (h=0;h<32;h++) { \
                if ((video_ram[v][h] >= TILE321_ID) && (video_ram[v][h] <= TILE324_ID)) \
                    video_ram[v][h] += 6; \
            } \
        } \
\
        hs_tx(sizeof(video_ram) / 4, (int32_t)video_ram, 0, GPU_VIDEO_RAM); \
        spi_write((room[0] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
    }

#define TOUCH_LEVER \
    if (((video_ram[y + 1][x] >= TILE309_ID) && (video_ram[y + 1][x] <= TILE312_ID)) || ((video_ram[y + 1][x + 1] >= TILE309_ID) && (video_ram[y + 1][x + 1] <= TILE312_ID))) { \
        for (v = 0; v < 22; v++) { \
            for (h = 0; h < 32; h++) { \
                if ((video_ram[v][h] == TILE309_ID)) { \
                    video_ram[v][h] = 0; \
                    video_ram[v][h+1] = TILE314_ID; \
                    h++; \
                } \
                else if ((video_ram[v][h] == TILE311_ID)) \
                    video_ram[v][h] = TILE315_ID; \
                else if ((video_ram[v][h] == TILE312_ID)) \
                    video_ram[v][h] = TILE316_ID; \
            } \
        } \
        hs_tx(sizeof(video_ram) / 4, (int32_t)video_ram, 0, GPU_VIDEO_RAM); \
        spi_write((room[0] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE); \
        if (room[0] == 9) \
            jean.flags[3] = 1; \
        if (room[0] == 10) \
            jean.flags[2] = 1; \
        if (room[0] == 20) \
            jean.flags[4] = 1; \
        waitcnt(CNT + _CLKFREQ * 2); \
    }

#define DRAW_SHOTS \
    /* Shoots from skeletons & gargoyles */ \
    for (n = 0; n <= 4; n += 2) { \
        if (proyec[n] > 0) { \
            i = proyec[n + 1]; \
\
            /* Move shoot */ \
            if (enemies[i].direction == 1) { \
                if (proyec[n] > enemies[i].limleft) \
                    proyec[n] -= INC_2_5; \
                else { \
                    enemies[i].fire = 0; \
                    enemies[i].speed = 0; \
                    proyec[n] = 0; \
                } \
            } else { \
                if (proyec[n] < enemies[i].limright) \
                    proyec[n] += INC_2_5; \
                else { \
                    enemies[i].fire = 0; \
                    enemies[i].speed = 0; \
                    proyec[n] = 0; \
                } \
            } \
\
            if (proyec[n] != 0) { \
                sprite0 = (((enemies[i].y & 0x1FF00) + INC_8_0) << 1) | ((proyec[n] & 0x1FF00) >> 8); \
                if (enemies[i].direction == 0) \
                    sprite0 |= MIRROR_FLAG; \
                sprite_ram[sprite_idx++] = sprite0; \
                sprite_ram[sprite_idx++] = sprite1; \
            } \
        } \
    }

void room_05_06_drawenemies();
void room_17_drawenemies();
void room_20_drawenemies();
void room_21_drawenemies();
void room_22_drawenemies();
void room_24_drawenemies();

void room_02_touchobj();
void room_03_touchobj();
void room_05_touchobj();
void room_06_touchobj();
void room_08_touchobj();
void room_09_touchobj();
void room_10_touchobj();
void room_11_touchobj();
void room_12_touchobj();
void room_13_touchobj();
void room_14_touchobj();
void room_15_touchobj();
void room_16_touchobj();
void room_17_touchobj();
void room_18_touchobj();
void room_19_touchobj();
void room_20_touchobj();
void room_21_touchobj();
void room_22_touchobj();
void room_23_touchobj();
void room_24_touchobj();

void room_07_events();
void room_08_events();
void room_10_events();
void room_19_events();
void room_20_events();
void room_24_events();

#endif /* OVERLAY_H_ */
