/**
 * Abbaye des Morts for Parallax Propeller
 * Copyright (c) 2015 Marco Maccaferri
 *
 * TERMS OF USE: MIT License
 */

#include <propeller.h>

#include "game.h"
#include "overlay.h"

static void showparchment() {
    memset(sprite_ram, 0, sizeof(sprite_ram));

    hs_tx(sizeof(sprite_ram) / 4, (int32_t)sprite_ram, 0, GPU_SPRITE_RAM);
    hs_tx(sizeof(mailbox) / 4, (int32_t)mailbox, 3, GPU_MAILBOX);
    waitcnt(CNT + _CLKFREQ * 5);

    hs_tx(sizeof(sprite_ram) / 4, (int32_t)sprite_ram, 5, GPU_SPRITE_RAM);
    hs_tx(sizeof(video_ram) / 4, (int32_t)video_ram, 0, GPU_VIDEO_RAM);
}

void plants() {
    int n = 0;
    int r = 0;

    common_drawenemies();

    /* Animation */

    for (n = 1; n < 4; n++) {
        if (enemies[n].speed < (140 + ((n - 1) * 30)))
            enemies[n].speed++;
        else
            enemies[n].speed = 0;

        if (enemies[n].speed < (100 + ((n - 1) * 30)))
            enemies[n].animation = 0;
        else
            enemies[n].animation = 1;

        sprite_ram[sprite_idx++] = ((enemies[n].y & 0x1FF00) << 1) | ((enemies[n].x & 0x1FF00) >> 8);
        sprite_ram[sprite_idx++] = GPU_TILES_RAM + ROOM_14_SPRITE_384_40_OFS + (128 * enemies[n].animation);
    }

    /* Init fire */
    for (n = 1; n < 4; n++) {
        if (enemies[n].speed == 135 + ((n - 1) * 30)) {
            r = (n - 1) * 4;
            proyec[r] = enemies[n].x - INC_1_0;
            proyec[r + 1] = enemies[n].y - INC_1_0;
            proyec[r + 2] = enemies[n].x + INC_16_0;
            proyec[r + 3] = enemies[n].y - INC_1_0;
            //Mix_PlayChannel(-1, fx[0], 0);
        }
    }

    /* Move fires */
    for (n = 1; n < 4; n++) {
        r = (n - 1) * 4;
        if (proyec[r] > 0) {
            if (enemies[n].direction == 0) {
                if (enemies[n].y - proyec[r + 1] < (24 << 8)) {
                    proyec[r + 1] -= INC_1_0;
                    proyec[r + 3] -= INC_1_0;
                }
                else
                    enemies[n].direction = 1;
                if ((counter[0] % 2) == 0) {
                    proyec[r] -= INC_1_0;
                    proyec[r + 2] += INC_1_0;
                }
            }
            if ((enemies[n].direction > 0) && (enemies[n].direction < 6)) {
                proyec[r] -= INC_1_0;
                proyec[r + 2] += INC_1_0;
                enemies[n].direction++;
            }
            if (enemies[n].direction == 6) {
                if (proyec[r + 1] < enemies[n].limleft) {
                    proyec[r + 1] += INC_2_0;
                    if (counter[0] % 2 == 0)
                        proyec[r] -= INC_1_0;
                }
                else {
                    proyec[r] = 0;
                    proyec[r + 1] = 0;
                }
                if ((proyec[r + 3] < enemies[n].limright) && (proyec[r + 3] > (7 << 8)) && (proyec[r + 2] < (240 << 8))) {
                    proyec[r + 3] += INC_2_0;
                    if (counter[0] % 2 == 0)
                        proyec[r + 2] += INC_1_0;
                }
                else {
                    proyec[r + 2] = 0;
                    proyec[r + 3] = 0;
                }
            }
            if ((proyec[r] == 0) && (proyec[r + 2] == 0))
                enemies[n].direction = 0;
        }

    }

    /* Draw fire */
    for (n = 1; n < 4; n++) {
        r = (n - 1) * 4;
        if (proyec[r] > 0 && proyec[r] < (256 << 8)) {
            sprite_ram[sprite_idx++] = ((proyec[r + 1] & 0x1FF00) << 1) | ((proyec[r] & 0x1FF00) >> 8);
            sprite_ram[sprite_idx++] = GPU_TILES_RAM + ROOM_14_SPRITE_680_32_OFS;
        }
        if (proyec[r + 2] > 0 && proyec[r + 2] < (256 << 8)) {
            sprite_ram[sprite_idx++] = ((proyec[r + 3] & 0x1FF00) << 1) | ((proyec[r + 2] & 0x1FF00) >> 8);
            sprite_ram[sprite_idx++] = GPU_TILES_RAM + ROOM_14_SPRITE_680_32_OFS;
        }
    }
}

void room_14_touchobj()
{
    int x = 0;
    int y = 0;
    int h = 0;
    int v = 0;

    x = (jean.x + INC_2_0) / 0x800;
    y = jean.y / 0x800;

    TOUCH_CROSSES
    TOUCH_HEARTS
    TOUCH_CHECKPOINT

    /* Touch yellow parchment */
    if (((video_ram[y+1][x] >= TILE317_ID) && (video_ram[y+1][x] <= TILE320_ID)) || ((video_ram[y+1][x+1] >= TILE317_ID) && (video_ram[y+1][x+1] <= TILE320_ID))) {
        for (v=0;v<22;v++) {
            for (h=0;h<32;h++) {
                if ((video_ram[v][h] >= TILE317_ID) && (video_ram[v][h] <= TILE320_ID))
                    video_ram[v][h] = 0;
            }
        }

        mailbox[0] = GPU_PARCHMENT4_TILES_RAM;
        mailbox[1] = PARCHMENT4_TILESET_SIZE;
        mailbox[2] = GPU_PARCHMENT4_TILEMAP;
        showparchment();
        spi_write((room[0] - 1) * (MAP_SIZE), (uint8_t *)&video_ram[0][0], MAP_SIZE);
    }
}
