#
# Abbaye des Morts for Parallax Propeller
# Tile and Sprite VGA Scanline Renderer
#
# Copyright (c) 2015 Marco Maccaferri
# MIT Licensed
#

                    .pasm
                    .compress off

                    .section .cogvideo_renderer, "ax"

                    .equ     hres, 256
                    .equ     vres, 192
                    .equ     hoffs, 32
                    .equ     voffs, 24
                    .equ     cogs, 4

                    .org     0

driver              jmp     #begin

# driver parameters

offset              long    0                       ' $0001
base                long    0                       ' $0002

begin               mov     tile_palette_ptr, #tile_palette
                    shl     tile_palette_ptr, #16
                    or      tile_palette_ptr, #tile_palette

                    mov     scnt, #vres-cogs
                    add     scnt, offset

                    rdlong  a, blnk
                    cmp     a, scnt wz
            if_nz   jmp     #$-2                    ' synch with last line

                    add     scnt, #cogs             ' next line to render

vsync
                    ' read sprite status

                    mov     sbuf_ptr, sprite_ram
                    movd    _rd0, #sprite11 +1
                    movd    _rd1, #sprite11
                    add     sbuf_ptr, #24 * 4 -1
                    movi    sbuf_ptr, #24 - 2
_rd0                rdlong  0-0, sbuf_ptr
                    sub     _rd0, inc_dest_2
                    sub     sbuf_ptr, i2s7 wc
_rd1                rdlong  0-0, sbuf_ptr
                    sub     _rd1, inc_dest_2
            if_nc   djnz    sbuf_ptr, #_rd0

loop

                    cmp     scnt, #voffs+vres wz,wc
            if_nc   jmp     #_blank
                    cmp     scnt, #voffs wz,wc
            if_nc   jmp     #active

_blank              mov     a, #0
                    movd    _l1, #sbuf
                    mov     ecnt, #hres/4
_l1                 mov     0-0, a
                    add     _l1, inc_dest
                    djnz    ecnt, #_l1

                    jmp     #emit+1

active              sub     scnt, #voffs

                    mov     video_ptr, scnt
                    shr     video_ptr, #3
                    shl     video_ptr, #6
                    add     video_ptr, video_ram

                    mov     loffs, scnt
                    and     loffs, #7
                    shl     loffs, #2
                    add     loffs, tiles_rom

                    mov     ccnt, #0
                    mov     ecnt, #hres/8
                    movd    str0, #sbuf
                    movd    str1, #sbuf+1

rloop               rdword  tile_ptr, video_ptr     ' read tile number to display

                    mov     palette_ptr, tile_ptr
                    and     palette_ptr, palettemask
                    shr     palette_ptr, #6

                    and     tile_ptr, tilemask
                    shl     tile_ptr, #5            ' 32 bytes per tile
                    add     tile_ptr, loffs
                    rdlong  tile, tile_ptr          ' pixels, 4 bit per pixel, from msb

                    mov     a, palette_ptr
                    shl     a, #16
                    or      palette_ptr, a
                    add     palette_ptr, tile_palette_ptr

                    mov     a, tile
                    and     a, mask_04
                    shr     a, #12
                    add     a, palette_ptr
                    movs    mv4, a
                    shr     a, #16
                    movs    mv0, a

                    mov     a, tile
                    and     a, mask_15
                    shr     a, #8
                    add     a, palette_ptr
                    movs    mv5, a
                    shr     a, #16
                    movs    mv1, a

                    mov     a, tile
                    and     a, mask_26
                    shr     a, #4
                    add     a, palette_ptr
                    movs    mv6, a
                    shr     a, #16
                    movs    mv2, a

                    and     tile, mask_37           ' last pixel pair, tile not needed anymore
                    add     tile, palette_ptr
                    movs    mv7, tile
                    shr     tile, #16
                    movs    mv3, tile

mv7                 mov     pixels2, 0-0
                    shl     pixels2, #8
mv6                 or      pixels2, 0-0
                    shl     pixels2, #8
mv5                 or      pixels2, 0-0
                    shl     pixels2, #8
mv4                 or      pixels2, 0-0

mv3                 mov     pixels1, 0-0
                    shl     pixels1, #8
mv2                 or      pixels1, 0-0
                    shl     pixels1, #8
mv1                 or      pixels1, 0-0
                    shl     pixels1, #8
mv0                 or      pixels1, 0-0

str0                mov     0-0, pixels1
                    add     str0, inc_dest_2
str1                mov     0-0, pixels2
                    add     str1, inc_dest_2

                    add     ccnt, #8
                    add     video_ptr, #2
                    djnz    ecnt, #rloop

                    cmp     scnt, #vres-16 wz,wc        ' no sprites below vres-16 (status bar)
            if_nc   jmp     #emit

                    mov     tile, sprite0
                    mov     tile_ptr, sprite0 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite1
                    mov     tile_ptr, sprite1 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite2
                    mov     tile_ptr, sprite2 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite3
                    mov     tile_ptr, sprite3 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite4
                    mov     tile_ptr, sprite4 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite5
                    mov     tile_ptr, sprite5 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite6
                    mov     tile_ptr, sprite6 + 1 wz
            if_nz   call    #draw_sprite
                    
                    mov     tile, sprite7
                    mov     tile_ptr, sprite7 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite8
                    mov     tile_ptr, sprite8 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite9
                    mov     tile_ptr, sprite9 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite10
                    mov     tile_ptr, sprite10 + 1 wz
            if_nz   call    #draw_sprite

                    mov     tile, sprite11
                    mov     tile_ptr, sprite11 + 1 wz
            if_nz   call    #draw_sprite

emit                add     scnt, #voffs

                    rdlong  a, blnk
                    cmp     a, scnt wz
            if_nz   jmp     #$-2                    ' wait for line fetch start

{
                    rdlong  a, blnk
                    cmp     a, scnt wz
            if_z    jmp     #$-2                    ' wait for line fetch end
}

                    mov     sbuf_ptr, base
                    movd    _wr0, #sbuf_ptr-1       ' last long in cog buffer
                    movd    _wr1, #sbuf_ptr-2       ' second-to-last long in cog buffer
                    add     sbuf_ptr, #64 * 4 -1    ' last byte in hub buffer (8n + 7)
                    movi    sbuf_ptr, #64 - 2       ' add magic marker
_wr0                wrlong  0-0, sbuf_ptr           ' |
                    sub     _wr0, inc_dest_2        ' |
                    sub     sbuf_ptr, i2s7 wc       ' |
_wr1                wrlong  0-0, sbuf_ptr           ' |
                    sub     _wr1, inc_dest_2        ' |
            if_nc   djnz    sbuf_ptr, #_wr0         ' sub #7/djnz

                    add     scnt, #cogs             ' next line to render
                    cmpsub  scnt, #voffs+vres+voffs wz,wc
            if_c    jmp     #vsync

                    jmp     #loop

draw_sprite         mov     y, tile                 ' y-position mask
                    shl     y, #14
                    sar     y, #23                  ' sign-extend y

                    mov     b, tile_ptr             ' calculate height
                    shr     b, #19
                    and     b, #$38

                    mov     a, scnt                 ' check sprite scanline visibility
                    subs    a, y  wc,wz
            if_c    jmp     #draw_sprite_ret
                    cmp     a, b wc,wz
            if_nc   jmp     #draw_sprite_ret

                    mov     b, tile_ptr             ' set palette pointer
                    shr     b, #25
                    mov     palette_ptr, b
                    shl     palette_ptr, #16
                    or      palette_ptr, b
                    add     palette_ptr, tile_palette_ptr

                    mov     b, tile_ptr             ' calculate width
                    shr     b, #16
                    and     b, #$38

                    shl     a, #2
                    add     tile_ptr, a
                    cmp     b, #16 wc
            if_nc   add     tile_ptr, a
            if_nc   cmp     b, #24 wc
            if_nc   add     tile_ptr, a
            if_nc   cmp     b, #32 wc
            if_nc   add     tile_ptr, a

                    mov     ecnt, b                 ' width counter

                    sub     b, #8
                    shr     b, #1
                    test    tile, mirror_mask wc
        if_c        add     tile_ptr, b
                    
                    mov     x, tile                 ' x-position mask
                    shl     x, #23
                    sar     x, #23                  ' sign-extend x

                    movs    _src0, #sbuf            ' sets source and destination buffer pointers
                    movs    _src1, #sbuf+1
                    movd    _dst0, #sbuf
                    movd    _dst1, #sbuf+1

                    mov     a, x                    ' adjust scanline buffer pointer to x location
                    and     a, #$1F8
                    shl     a, #23
                    sar     a, #25                  ' sign-extend x
                    add     _src0, a
                    add     _src1, a
                    shl     a, #9
                    add     _dst0, a
                    add     _dst1, a

                    and     x, #$07
                    mov     ccnt, #8
                    sub     ccnt, x
                    shl     x, #2

                    mov     pixels2, #0
_l2                 rdlong  data, tile_ptr

                    test    tile, mirror_mask wc

                    mov     pixels1, data
            if_nc   shr     pixels1, x
            if_c    shl     pixels1, x
                    or      pixels1, pixels2

                    mov     a, #32
                    sub     a, x
                    mov     pixels2, data
            if_nc   shl     pixels2, a
            if_c    shr     pixels2, a

_src0               mov     colors1, 0-0
_src1               mov     colors2, 0-0

                    mov     acnt, #4
_l3         if_c    rol     pixels1, #4
                    mov     a, pixels1
                    and     a, mask_37
                    add     a, palette_ptr
                    movs    _st1, a
                    rol     colors2, #8
                    test    pixels1, mask_7 wz
            if_nz   andn    colors2, #$FF
_st1        if_nz   or      colors2, 0-0
                    shr     a, #16
                    movs    _st2, a
                    rol     colors1, #8
                    test    pixels1, mask_3 wz
            if_nz   andn    colors1, #$FF
_st2        if_nz   or      colors1, 0-0
            if_nc   shr     pixels1, #4
                    djnz    acnt, #_l3

_dst0               mov     0-0, colors1
_dst1               mov     0-0, colors2

                    add     _src0, #2
                    add     _src1, #2
                    add     _dst0, inc_dest_2
                    add     _dst1, inc_dest_2

            if_nc   add     tile_ptr, #4
            if_c    sub     tile_ptr, #4

                    cmp     x, #0 wz
            if_z    mov     pixels2, #0
                    mov     data, #0

                    sub     ecnt, ccnt  wc,wz
                    mov     ccnt, #8
        if_z_or_c   jmp     #draw_sprite_ret
                    cmp     ecnt, #8  wc,wz
            if_nc   jmp     #_l2
                    jmp     #_l2+1

draw_sprite_ret     ret


' driver parameters

blnk                long    @_vsync
video_ram           long    @_video_ram
tiles_rom           long    @_tiles_ram
sprite_ram          long    @_sprite_ram

' initialised data and/or presets

inc_dest            long    1 << 9
inc_dest_2          long    2 << 9
i2s7                long    2 << 23 | 7
mask_04             long    $F_0_0_0_F_0_0_0
mask_15             long    $0_F_0_0_0_F_0_0
mask_26             long    $0_0_F_0_0_0_F_0
mask_37             long    $0_0_0_F_0_0_0_F
mask_3              long    $0_0_0_F_0_0_0_0
mask_7              long    $0_0_0_0_0_0_0_F

tilemask            long    %0_00000_1111111111
palettemask         long    %0_11111_0000000000

mirror_mask         long    %0000001_0000000_000000000_000000000

pixels1             long    0
pixels2             long    0
colors1             long    0
colors2             long    0

tile_palette        .incbin "vga_palette.dat"

tile_palette_ptr    long    0 ' (tile_palette << 16) | tile_palette

' uninitialised data and/or temporaries

a                   res     1
b                   res     1
x                   res     1
y                   res     1
data                res     1                       ' data
tile                res     2

acnt                res     1                       ' attribute count
ecnt                res     1                       ' element count
lcnt                res     1                       ' line counter
scnt                res     1                       ' scanlines
ccnt                res     1                       ' column counter
loffs               res     1                       ' line offset

tile_ptr            res     1
video_ptr           res     1
palette_ptr         res     1

sprite0             res     2
sprite1             res     2
sprite2             res     2
sprite3             res     2
sprite4             res     2
sprite5             res     2
sprite6             res     2
sprite7             res     2
sprite8             res     2
sprite9             res     2
sprite10            res     2
sprite11            res     2

                    res     4
sbuf                res     64
sbuf_ptr            res     4

                    fit     $1F0


                    .data

                    .global _sprite_ram
                    .global _video_ram

_sprite_ram         res     12 * 2
_video_ram          res     (((hres / 8) * (vres / 8)) * 2) / 4

